# coding=utf-8

import time
from pocounit.case import PocoTestCase
from pocounit.addons.poco.action_tracking import ActionTracker

from poco.drivers.unity3d import UnityPoco


class TutorialCase(PocoTestCase):
    @classmethod
    def setUpClass(cls):
        from airtest.core.api import connect_device
        connect_device('Android:///')

        cls.poco = UnityPoco()
        action_tracker = ActionTracker(cls.poco)
        cls.register_addon(action_tracker)

    @classmethod
    def tearDownClass(cls):
        time.sleep(1)



# coding=utf-8
from __future__ import unicode_literals

from functools import wraps

from hrpc.exceptions import RpcRemoteException
from poco.exceptions import PocoTargetRemovedException


def transform_node_has_been_removed_exception(func):
    """
    将HRpcRemoteException.NodeHasBeenRemovedException转换成PocoTargetRemovedException

    :param func: 仅限getattr和setattr两个接口方法
    :return: 
    """

    @wraps(func)
    def wrapped(self, nodes, name, *args, **kwargs):
        """

        :param self:
        :param nodes: UI object proxy
        :param name: attribute name
        :param args:
        :param kwargs:
        :return:
        """
        try:
            return func(self, nodes, name, *args, **kwargs)
        except RpcRemoteException as e:
            if e.error_type == 'NodeHasBeenRemovedException' or e.error_type.endswith('.NodeHasBeenRemovedException'):
                raise PocoTargetRemovedException('{}: {}'.format(func.__name__, name), nodes)
            else:
                raise
    return wrapped



# coding=utf-8

from poco.utils.vector import Vec2

__all__ = ['MotionTrack', 'MotionTrackBatch']


def track_sampling(track, accuracy=0.002):
    # accuracy： 采样精度，都是归一化坐标系
    if len(track) <= 1:
        return track

    total_distance = 0.0
    for i in range(len(track) - 1):
        p0 = Vec2(track[i])
        p1 = Vec2(track[i + 1])
        total_distance += (p1 - p0).length

    sample_points = []
    for i in range(len(track) - 1):
        p0 = Vec2(track[i])
        p1 = Vec2(track[i + 1])
        d = p1 - p0
        seg_length = d.length
        d = (p1 - p0).unit() * accuracy
        sp = p0
        while (sp - p0).length < seg_length:
            sample_points.append(sp.to_list())
            sp = sp + d

    sample_points.append(track[-1])
    return sample_points


class MotionTrackHold(object):
    def __init__(self, how_long):
        super(MotionTrackHold, self).__init__()
        self.how_long = how_long


class MotionTrack(object):
    def __init__(self, points=None, speed=0.4):
        super(MotionTrack, self).__init__()
        self.speed = speed
        self.timestamp = 0
        self.event_points = []  # [ts, (x, y), contact_id],  timestamp as arrival time

        if points:
            for p in points:
                self.move(p)

    @property
    def last_point(self):
        if self.event_points:
            return self.event_points[-1][1]
        return None

    def start(self, p):
        return self.move(p)

    def move(self, p):
        if self.last_point:
            dt = (Vec2(p) - Vec2(self.last_point)).length / self.speed
            self.timestamp += dt
        self.event_points.append([self.timestamp, p, 0])
        return self

    def hold(self, t):
        self.timestamp += t
        if self.event_points:
            self.move(self.last_point)
        return self

    def set_contact_id(self, _id):
        for ep in self.event_points:
            ep[2] = _id

    def discretize(self, contact_id=0, accuracy=0.004, dt=0.001):
        """
        Sample this motion track into discretized motion events.

        Args:
            contact_id: contact point id
            accuracy: motion minimum difference in space
            dt: sample time difference
        """

        if not self.event_points:
            return []

        events = []
        action_dt = accuracy / self.speed
        dt = dt or action_dt

        ep0 = self.event_points[0]
        for _ in range(int(ep0[0] / dt)):
            events.append(['s', dt])
        events.append(['d', ep0[1], contact_id])
        for i, ep in enumerate(self.event_points[1:]):
            prev_ts = self.event_points[i][0]
            curr_ts = ep[0]
            p0 = self.event_points[i][1]
            p1 = ep[1]
            if p0 == p1:
                # hold
                for _ in range(int((curr_ts - prev_ts) / dt)):
                    events.append(['s', dt])
            else:
                # move
                dpoints = track_sampling([p0, p1], accuracy)
                for p in dpoints:
                    events.append(['m', p, contact_id])
                    for _ in range(int(action_dt / dt)):
                        events.append(['s', dt])

        events.append(['u', contact_id])
        return events


class MotionTrackBatch(object):
    def __init__(self, tracks):
        super(MotionTrackBatch, self).__init__()
        self.tracks = tracks

    def discretize(self, accuracy=0.004):
        if accuracy < 0.001:
            accuracy = 0.001
        events = []
        discretized_tracks = [t.discretize(i, accuracy) for i, t in enumerate(self.tracks)]

        while discretized_tracks:
            # take motion events
            for dtrack in discretized_tracks:
                while True:
                    evt = dtrack[0]
                    if evt[0] != 's':
                        events.append(evt)
                        dtrack.pop(0)
                    else:
                        break
                    if not dtrack:
                        break

            discretized_tracks = list(filter(lambda a: a != [], discretized_tracks))

            # take sleep events
            while discretized_tracks and all(dtrack[0][0] == 's' for dtrack in discretized_tracks):
                evt_sleep = discretized_tracks[0][0]
                if events:
                    prev_evt = events[-1]
                    if prev_evt[0] == 's':
                        prev_evt[1] += evt_sleep[1]
                        events[-1] = prev_evt
                    else:
                        events.append(evt_sleep)
                else:
                    events.append(evt_sleep)
                for dtrack in discretized_tracks:
                    dtrack.pop(0)

                discretized_tracks = list(filter(lambda a: a != [], discretized_tracks))

        return events



# coding=utf-8

import time
from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class OverviewTutorial(TutorialCase):
    def runTest(self):
        self.poco('btn_start').click()
        time.sleep(1)

        self.poco(textMatches='drag.*').click()
        time.sleep(1)

        shell = self.poco('shell').focus('center')
        for star in self.poco('star'):
            star.drag_to(shell)
        time.sleep(1)

        self.assertEqual(self.poco('scoreVal').get_text(), "100", "score correct.")
        self.poco('btn_back', type='Button').click()

    def tearDown(self):
        time.sleep(2)


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-14 19:47:51
import json

from poco import Poco
from poco.agent import PocoAgent
from poco.utils.simplerpc.utils import sync_wrapper
from poco.freezeui.hierarchy import FrozenUIHierarchy, FrozenUIDumper
from poco.utils.airtest import AirtestInput, AirtestScreen
from poco.utils.simplerpc.rpcclient import RpcClient
from poco.utils.simplerpc.transport.ws import WebSocketClient
from poco.utils import six

from airtest.core.api import connect_device, device as current_device
from airtest.core.helper import device_platform


__all__ = ['CocosJsPoco']
DEFAULT_ADDR = "ws://localhost:5003"


class CocosJsPocoAgent(PocoAgent):
    def __init__(self, addr=DEFAULT_ADDR):
        try:
            port = int(addr.rsplit(":", 1)[-1])
        except ValueError:
            raise ValueError('Argument `addr` should be a string-like format. e.g. "ws://192.168.1.2:5003". Got {}'
                             .format(repr(addr)))

        if not current_device():
            connect_device("Android:///")
        if device_platform() == 'Android':
            local_port, _ = current_device().adb.setup_forward('tcp:{}'.format(port))
            ip = 'localhost'
            port = local_port
        else:
            import socket
            ip = socket.gethostbyname(socket.gethostname())
            # Note: ios is not support for now.

        self.conn = WebSocketClient('ws://{}:{}'.format(ip, port))
        self.c = RpcClient(self.conn)
        self.c.DEBUG = False
        # self.c.run(backend=True)
        self.c.wait_connected()

        hierarchy = FrozenUIHierarchy(Dumper(self.c))
        screen = AirtestScreen()
        input = AirtestInput()
        super(CocosJsPocoAgent, self).__init__(hierarchy, input, screen, None)


class Dumper(FrozenUIDumper):
    def __init__(self, rpcclient):
        super(Dumper, self).__init__()
        self.rpcclient = rpcclient

    @sync_wrapper
    def dumpHierarchy(self, onlyVisibleNode=True):
        return self.rpcclient.call("dump", onlyVisibleNode)


class CocosJsPoco(Poco):
    """docstring for CocosJsPoco"""

    def __init__(self, addr=DEFAULT_ADDR, **options):
        agent = CocosJsPocoAgent(addr)
        if 'action_interval' not in options:
            options['action_interval'] = 0.5
        super(CocosJsPoco, self).__init__(agent, **options)


# test code
def dump():
    from websocket import create_connection
    ws = create_connection(DEFAULT_ADDR, timeout=2)
    ws.send('{"jsonrpc": "2.0", "params": {}, "method": "dump", "id": 0}')
    # print("Sent")
    # print("Receiving...")
    result = ws.recv()
    # print("Received '%s'" % result)
    ws.close()
    data = json.loads(result)
    return data["result"]



# coding=utf-8

import time
from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class LocalPositioning1Tutorial(TutorialCase):
    def runTest(self):
        # focus is immutable
        fish = self.poco('fish').child(type='Image')
        fish_right_edge = fish.focus([1, 0.5])
        fish.long_click()  # still click the center
        time.sleep(0.2)
        fish_right_edge.long_click()  # will click the right edge
        time.sleep(0.2)


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# coding=utf-8

__author__ = 'lxn3032'
__all__ = ['IDumper', 'AbstractDumper']


class IDumper(object):
    """
    This interface defines the standard dumper behavior. Dumper class is introduced to get the hierarchy information and
    convert it into serializable data.
    """

    def getRoot(self):
        """
        Return the root node of the UI Hierarchy. The node information is wrapped by
        :py:class:`AbstractNode <poco.sdk.AbstractNode>`.
        See definition of :py:class:`AbstractNode <poco.sdk.AbstractNode>` for more details.

        Returns:
            :py:class:`inherit from AbstractNode <poco.sdk.AbstractNode>`: instance that holds the hierarchy data
        """

        raise NotImplementedError

    def dumpHierarchy(self, onlyVisibleNode):
        """
        Return the json serializable dictionary holding the hierarchy data. Refer to sample of returned structure object
        below.

        Structure of the dict::

            {
                # name can be duplicated from the original name or just left the default one
                # if it cannot be determined, however providing some meaningful name is preferred
                'name': '<a recognizable string>'
                
                # All available attributes of this node are in form of key-value pairs
                'payload': {
                    'name': '',
                    'pos': [0, 0],
                    'size': [1, 1],
                    ...
                },
                
                # If there is no child, this part can be omitted
                'children': [
                    {...},  # Same structure as this dict. 
                ],
            }

        Returns:
            :obj:`dict` or :obj:`NoneType`: hierarchy data or None
        """

        raise NotImplementedError


class AbstractDumper(IDumper):
    """
    This class partially implements ``IDumper`` using general traversal algorithm. In order to dump the hierarchy from
    the root node, this dumper first retrieves all available attributes of the root node and also the list all its
    children and then applies the same procedures as described on each child (i.e. treats each child as a root node)
    until the node that has no child(ren) is reached.
    """

    def dumpHierarchy(self, onlyVisibleNode=True):
        """
        Returns:
            :obj:`dict`: json serializable dict holding the whole hierarchy data
        """

        return self.dumpHierarchyImpl(self.getRoot(), onlyVisibleNode)

    def dumpHierarchyImpl(self, node, onlyVisibleNode=True):
        """
        Crawl the hierarchy tree using the simple BFS algorithm. The ``dump`` procedure is the engine independent as
        the hierarchy structure is wrapped by :py:class:`AbstractNode <poco.sdk.AbstractNode>` and therefore the
        ``dump`` procedure can be algorithmized.

        Following code demonstrates the simplest implementation. Feel free to implement your own algorithms to
        optimize the performance.

        .. note:: Do not explicitly call this method as this is an internal function, call
                  :py:meth:`dumpHierarchy() <poco.sdk.AbstractDumper.AbstractDumper.dumpHierarchy>` function instead
                  if you want to dump the hierarchy.

        Args:
            node(:py:class:`inherit from AbstractNode <poco.sdk.AbstractNode>`): root node of the hierarchy to be
             dumped
            onlyVisibleNode(:obj:`bool`): dump only the visible nodes or all nodes, default to True

        Returns:
            :obj:`dict`: json serializable dict holding the whole hierarchy data
        """

        if not node:
            return None

        payload = {}

        # filter out all None values
        for attrName, attrVal in node.enumerateAttrs():
            if attrVal is not None:
                payload[attrName] = attrVal

        result = {}
        children = []
        for child in node.getChildren():
            if not onlyVisibleNode or child.getAttr('visible'):
                children.append(self.dumpHierarchyImpl(child, onlyVisibleNode))
        if len(children) > 0:
            result['children'] = children

        result['name'] = payload.get('name') or node.getAttr('name')
        result['payload'] = payload

        return result



from __future__ import absolute_import

from django.views.decorators.csrf import csrf_exempt
from django.conf.urls import url
from django.http import HttpResponse, HttpResponseNotAllowed
import copy
import json
import logging
import time

from ..exceptions import JSONRPCInvalidRequestException
from ..jsonrpc import JSONRPCRequest
from ..manager import JSONRPCResponseManager
from ..utils import DatetimeDecimalEncoder
from ..dispatcher import Dispatcher


logger = logging.getLogger(__name__)


class JSONRPCAPI(object):
    def __init__(self, dispatcher=None):
        self.dispatcher = dispatcher if dispatcher is not None \
            else Dispatcher()

    @property
    def urls(self):
        urls = [
            url(r'^$', self.jsonrpc),
            url(r'map$', self.jsonrpc_map),
        ]

        return urls

    @csrf_exempt
    def jsonrpc(self, request):
        """ JSON-RPC 2.0 handler."""
        if request.method != "POST":
            return HttpResponseNotAllowed(["POST"])

        request_str = request.body.decode('utf8')
        try:
            jsonrpc_request = JSONRPCRequest.from_json(request_str)
        except (TypeError, ValueError, JSONRPCInvalidRequestException):
            response = JSONRPCResponseManager.handle(
                request_str, self.dispatcher)
        else:
            jsonrpc_request.params = jsonrpc_request.params or {}
            jsonrpc_request_params = copy.copy(jsonrpc_request.params)
            if isinstance(jsonrpc_request.params, dict):
                jsonrpc_request.params.update(request=request)

            t1 = time.time()
            response = JSONRPCResponseManager.handle_request(
                jsonrpc_request, self.dispatcher)
            t2 = time.time()
            logger.info('{0}({1}) {2:.2f} sec'.format(
                jsonrpc_request.method, jsonrpc_request_params, t2 - t1))

        if response:
            def serialize(s):
                return json.dumps(s, cls=DatetimeDecimalEncoder)

            response.serialize = serialize
            response = response.json

        return HttpResponse(response, content_type="application/json")

    def jsonrpc_map(self, request):
        """ Map of json-rpc available calls.

        :return str:

        """
        result = "<h1>JSON-RPC map</h1><pre>{0}</pre>".format("\n\n".join([
            "{0}: {1}".format(fname, f.__doc__)
            for fname, f in self.dispatcher.items()
        ]))
        return HttpResponse(result)


api = JSONRPCAPI()
